#include <aros/bigendianio.h>
#include <aros/asmcall.h>

#include <proto/arossupport.h>
#include <proto/exec.h>
#include <clib/alib_protos.h>

#include <stdio.h>

/********************************************************************/

BOOL ReadByte(struct Hook *hook, UBYTE *dataptr, void *stream)
{
    LONG value = CallHook(hook, stream, BEIO_READ);
    
    if (value != EOF)
    	*dataptr = value;
	
    return (value != EOF);
}

/********************************************************************/

BOOL ReadWord(struct Hook *hook, UWORD *dataptr, void *stream)
{
    LONG value, c;
    
    /* High byte */
    c = CallHook (hook, stream, BEIO_READ);

    if (c == EOF)
	return FALSE;

    /* Low byte */
    value = CallHook (hook, stream, BEIO_READ);

    if (value == EOF)
	return FALSE;

    *dataptr = (c << 8) + value;
   	
    return TRUE;
}

/********************************************************************/

BOOL ReadLong(struct Hook *hook, ULONG *dataptr, void *stream)
{
    ULONG value;
    LONG c;
    UBYTE *ptr = (UBYTE *)&value;

#define READ_ONE_BYTE		    \
    if ((c = CallHook (hook, stream, BEIO_READ)) == EOF) \
	return FALSE;		    \
				    \
    *ptr++ = c
    
    READ_ONE_BYTE;
    READ_ONE_BYTE;
    READ_ONE_BYTE;
    READ_ONE_BYTE;

    *dataptr = value;
    
    return TRUE;
}

/********************************************************************/

BOOL ReadString(struct Hook *hook, STRPTR *dataptr, void *stream)
{
    STRPTR buffer;
    LONG   size, maxsize;
    LONG   c;

    size = maxsize = 0;
    buffer = NULL;

    for (;;)
    {
	c = CallHook (hook, stream, BEIO_READ);

	if (c == EOF)
	{
	    FreeVec (buffer);
	    return FALSE;
	}

	if (size == maxsize)
	{
	    STRPTR tmp;

	    tmp = AllocVec (maxsize + 16, MEMF_ANY);

	    if (!tmp)
	    {
		FreeVec (buffer);
		return FALSE;
	    }

	    if (buffer)
	    {
		memcpy (tmp, buffer, maxsize);
		FreeVec (buffer);
	    }

    	    maxsize += 16;
	    
	    buffer = tmp;
	}

	buffer[size ++] = c;

	if (!c)
	    break;
    }

    *dataptr = buffer;

    return TRUE;
}

/********************************************************************/

BOOL WriteByte(struct Hook *hook, UBYTE data, void * stream)
{
    return (CallHook(hook, stream, BEIO_WRITE, data) != EOF);
}

/********************************************************************/

BOOL WriteWord(struct Hook *hook, UWORD data, void * stream)
{
    if (CallHook (hook, stream, BEIO_WRITE, data >> 8) == EOF)
	return FALSE;

    return (CallHook (hook, stream, BEIO_WRITE, data & 0xFF) != EOF);
}

/********************************************************************/

BOOL WriteLong(struct Hook *hook, ULONG data, void * stream)
{
    UBYTE *ptr = (UBYTE *)&data;
    
#define WRITE_ONE_CHAR			\
    if (CallHook (hook, stream, BEIO_WRITE, *ptr++) == EOF) \
	return FALSE

    WRITE_ONE_CHAR;
    WRITE_ONE_CHAR;
    WRITE_ONE_CHAR;
    WRITE_ONE_CHAR;

    return TRUE;
}

/********************************************************************/

BOOL WriteString(struct Hook *hook, STRPTR data, void *stream)
{
    do
    {
	if (CallHook (hook, stream, BEIO_WRITE, *data) == EOF)
	    return FALSE;
    } while (*data ++);

    return TRUE;
}

/********************************************************************/

#ifndef __MORPHOS__
#define RawPutChar(chr) \
  LP1NR(0x204, RawPutChar, UBYTE, chr, d0, \
  , SysBase)
  
static AROS_UFH1(void, puttostr,
    AROS_UFHA(UBYTE, chr, D0))
{
    RawPutChar(chr);
}

int kprintf(char *fmt, ...)
{    
    RawDoFmt(fmt, &fmt + 1, (VOID_FUNC)puttostr, NULL);
    return 0;
}
#endif
/********************************************************************/
